/*-------------------------------------------------------------------*
  *    Name: canplot.sas                                             *
  *   Title: Canonical discriminant structure plot.                  *
 *         Plots class means on two canonical variables, confidence  *
 *         circles for those means, and variable vectors showing the *
 *         correlations of variables with the canonical variates.    *
 *    Doc: http://www.math.yorku.ca/SCS/sasmac/canplot.html          *
 *-------------------------------------------------------------------*
 *  Author:  Michael Friendly            <friendly@YorkU.ca>         *
 * Created:  24 Nov 1991 13:17:10                                    *
 * Revised:  30 Jan 2000 09:59:37                                    *
 * Version:  1.2                                                     *
 *  - Added hsym, legend control                                     *
 *  - Added canx= cany= to plot other canonical dimensions           *
 *  - Warning message for <3 groups (only 1 dimension) - no plot     *
 *-------------------------------------------------------------------*/
 
%macro canplot(
	data=_last_,  /* name of data set to analyze               */
	class=,       /* name of class variable                    */
	var=,         /* list of classification variables          */
	dim=2,        /* number of canonical dimensions */
	scale=4,      /* scale factor for variable vectors in plot */
	conf=.99,     /* confidence probability for canonical means*/
	out=_dscore_, /* output data set containing discrim scores */
	outvar=_coef_,
	anno=_danno_, /* output data set containing annotations    */
   annoadd=MEAN,  /* additional annotations           */
	plot=YES,     /* or NO to suppress plot                    */
	haxis=,       /* AXIS statement for horizontal axis        */
	vaxis=,       /* and for vertical axis- use to equate axes */
	legend=,      /* LEGEND statement                          */
	hsym=1.2,     /* height of plot symbols                    */
	canx=can1,    /* Horizontal axis of plot */
	cany=can2,    /* Vertical axis of plot */
	dimlab=Canonical Dim.,
	colors=RED GREEN BLUE BLACK PURPLE YELLOW BROWN ORANGE,
	symbols=+ square star -     plus   :      $     =,
	name=CANPLOT /* name for graphic catalog entry            */
	  );
 
%let plot = %upcase(&PLOT);
%let annoadd = %upcase(&annoadd);
%let legend = %upcase(&legend);
%if %length(&var)=0 %then %do;
    %put CANPLOT:  You must specify VAR= variables list ;
    %goto DONE;
    %end;
%if %length(&class)=0 or %length(%scan(&class,2))>0 %then %do;
    %put CANPLOT:  You must specify one CLASS= variable ;
    %goto DONE;
    %end;
 
proc candisc data=&data ncan=&dim
             out=&out short
             outstat=_dstat_;
   classes &class;
   var &var ;
	run;
%if &sysver > 6.08 %then %do;
	*-- Extract canonical correlations --> % eig;
	data _dull_;
		set _dstat_ end=eof;
		drop &class &var;
		where (_type_='CANCORR');
		array v{*} &var;
		r = v(1);
		eig = r**2 / (1-r**2);
		cum+eig;
		if eof then call symput('cum', left(put(cum,best12.5)));
	run;
	data _null_;
		set _dull_;
		p = eig / &cum;
		pc = '(' || trim(left(put(100*p, 4.1))) || '%)';
		call symput('p'||put(_n_,1.), pc);
	proc print;
	%end;
%else %do i=1 %to &dim;
	%let p&i=;
	%end;
proc sort data=&out;
   by &class;
data &out;
	set &out;
	label can1 = "&dimlab 1 &p1"
		can2 = "&dimlab 2 &p2";

proc means data=&out noprint;
   var &canx &cany;
   by &class;
   output out=_means_ mean=&canx &cany n=n;
 
data &anno;
   set _means_ end=eof;
   length text $16 color function $8;
   retain xsys '2' ysys '2';
   drop &canx &cany n;
   x = &canx;
   y = &cany;
   color=scan("&colors",_n_);
   /* mark the class mean        */
	%if %index(&annoadd,MEAN) > 0 %then %do;
		text = 'PLUS';
		hsys = '4'; size = 4;
		function = 'SYMBOL';      output;
		%end;
 
 *-- Write group label (convert numeric &class to character);
   %if &class ^= %str() and %index(&annoadd,GPLABEL) > 0 %then %do;
      text = left(&class);
      position='3';
      size = 1.4; hsys=' ';
      function='LABEL'; output;
   %end;

   /* draw confidence region */
   size = sqrt( cinv(&conf, 2, 0) / n ) ;   * radius ;
   line = 20;
   do a = 0 to 360 by 10;                   * draw a "circle" ;
      ang = a*arcos(-1)/180;                * convert to radians;
      xp=  size * cos(ang);
      yp=  size * sin(ang);
      x = xp+ &canx;                         * translate to means;
      y = yp+ &cany;
      if a=0 then FUNCTION = 'MOVE    ';
             else FUNCTION = 'DRAW    ';
      output;
   end;
	
   if eof then do;          * save number of groups ;
      call symput('NGP',put(_n_,best5.));
      end;
*proc print data=_dstat_;

data _coeff_;
   set _dstat_;             * get standardized coefficients ;
   drop _TYPE_ ;
   if _type_ = 'STRUCTUR' |  /* Version 6 */
      _type_ = 'TSTRUCT'  |  /* Version 5 */
		_type_ = 'RSQUARED';
	if _type_ = 'RSQUARED' then _name_='RSQUARED';

proc transpose out=&outvar;
proc print data=&outvar;

%if &ngp < 3 %then %do;
	%put WARNING:  CANPLOT cannot produce a plot for &NGP groups.;
	%goto done;
	%end;

data _vector_;
   set &outvar;
   where (&canx^=.);
   length function text $8;
   retain xsys '2' ysys '2' position '6';
   x = 0 ; y = 0;
   function = 'MOVE ' ; output;
   x = &scale * &canx ;
   y = &scale * &cany ;
   function = 'DRAW ' ; output;
   text = _NAME_;
	text = substr(text,1,1) || lowcase(substr(text,2));
	if &cany >=0                                                               
		then position='2';             /* up justify         */                
		else position='E';             /* down justify       */
   function = 'LABEL' ; output;
data &anno;
   set &anno _vector_;
 
%if &plot = YES %then %do;
   %gensym(n=&ngp, h=&hsym, symbols=&symbols, colors=&colors);

   %if %length(&haxis)=0 %then %do;
      axis2 label=(h=&hsym) value=(h=&hsym);
      %let haxis=axis2;
      %end;
   %if %length(&vaxis)=0 %then %do;
      axis1 label=(h=&hsym) value=(h=&hsym);
      %let vaxis=axis1;
      %end;

	%if &legend=NONE %then %let legend=nolegend;
   %else %if %length(&legend)=0 %then %do;
      legend1 label=(h=&hsym) value=(h=&hsym);
      %let legend=legend=legend1;
      %end;
	%else %let legend=legend=&legend;


   proc gplot data=&out ;
      plot &cany * &canx = &class
           / anno=&anno frame
             vaxis=&vaxis haxis=&haxis &legend
             hm=1 vm=1 name="&name"
				 des="canplot of &data" ;
      run; quit;
   %end;
proc datasets lib=work memtype=data nolist;
   delete _coeff_ _dstat_ _means_ _vector_;
%done:
%mend;
 
 /*----------------------------------------------------*
  |  Macro to generate SYMBOL statement for each GROUP |
  *----------------------------------------------------*/
%macro gensym(n=1, h=1.5, i=none,
              symbols=%str(- + : $ = X _ Y),
              colors=BLACK RED GREEN BLUE BROWN YELLOW ORANGE PURPLE);
    %*-- note: only 8 symbols & colors are defined;
    %*--  if more than 8 groups symbols and colors are recycled;
  %local chr col k;
  %do k=1 %to &n ;
     %if %length(%scan(&symbols, &k, %str( ))) = 0 
	      %then %let symbols = &symbols &symbols;
     %if %length(%scan(&colors, &k, %str( ))) = 0 
	      %then %let colors = &colors &colors;
     %let chr =%scan(&symbols, &k,%str( ));
     %let col =%scan(&colors, &k, %str( ));
     symbol&k h=&h v=&chr c=&col i=&i;
  %end;
%mend gensym;
